@@ -47,3 +47,9 @@ __all__ = ['DataElement',
            'write_file',
            '__version__',
            '__version_info__']
+
+from pydicom.compat import in_py2
+if in_py2:
+    import warnings
+    msg = 'Python 2 will no longer be supported after the pydicom v1.4 release'
+    warnings.warn(msg, DeprecationWarning)
