@@ -7,7 +7,6 @@ from __future__ import absolute_import
 from io import BytesIO
 import os
 from struct import (Struct, unpack)
-from sys import byteorder
 import warnings
 import zlib
 
@@ -18,7 +17,7 @@ from pydicom.compat import in_py2
 from pydicom.config import logger
 from pydicom.datadict import dictionary_VR, tag_for_keyword
 from pydicom.dataelem import (DataElement, RawDataElement,
-                              DataElement_from_raw)
+                              DataElement_from_raw, empty_value_for_VR)
 from pydicom.dataset import (Dataset, FileDataset)
 from pydicom.dicomdir import DicomDir
 from pydicom.errors import InvalidDicomError
@@ -32,92 +31,6 @@ from pydicom.util.hexutil import bytes2hex
 from pydicom.valuerep import extra_length_VRs
 
 
-class DicomIter(object):
-    """Iterator over DICOM data elements created from a file-like object"""
-
-    def __init__(self, fp, stop_when=None, force=False):
-        """Read the preamble and meta info and prepare
-           iterator for remainder of file.
-
-        Parameters
-        ----------
-        fp : an open DicomFileLike object, at start of file
-        force : boolean
-            Force reading of data. See ``read_file`` for
-             more parameter info.
-
-        Adds flags to fp: Big/Little-endian & Implicit/Explicit VR
-        """
-        self.fp = fp
-        self.stop_when = stop_when
-        self.preamble = preamble = read_preamble(fp, force)
-        self.has_header = has_header = (preamble is not None)
-        self.file_meta_info = Dataset()
-
-        if has_header:
-            self.file_meta_info = file_meta_info = _read_file_meta_info(fp)
-            transfer_syntax = file_meta_info.TransferSyntaxUID
-
-            if transfer_syntax == pydicom.uid.ExplicitVRLittleEndian:
-                self._is_implicit_VR = False
-                self._is_little_endian = True
-
-            elif transfer_syntax == pydicom.uid.ImplicitVRLittleEndian:
-                self._is_implicit_VR = True
-                self._is_little_endian = True
-
-            elif transfer_syntax == pydicom.uid.ExplicitVRBigEndian:
-                self._is_implicit_VR = False
-                self._is_little_endian = False
-
-            elif transfer_syntax == pydicom.uid.DeflatedExplicitVRLittleEndian:
-                # See PS3.6-2008 A.5 (p 71) -- when written, the entire dataset
-                # following the file metadata was prepared the normal way,
-                # then "deflate" compression applied.
-                # All that is needed here is to decompress and then
-                # use as normal in a file-like object
-                zipped = fp.read()
-
-                # -MAX_WBITS part is from comp.lang.python answer:
-                # groups.google.com/group/comp.lang.python/msg/e95b3b38a71e6799
-                unzipped = zlib.decompress(zipped, -zlib.MAX_WBITS)
-
-                # a file-like object
-                fp = BytesIO(unzipped)
-
-                # point to new object
-                self.fp = fp
-                self._is_implicit_VR = False
-                self._is_little_endian = True
-            else:
-                # Any other syntax should be Explicit VR Little Endian,
-                # e.g. all Encapsulated (JPEG etc) are ExplVR-LE
-                # by Standard PS 3.5-2008 A.4 (p63)
-                self._is_implicit_VR = False
-                self._is_little_endian = True
-
-        else:  # no header -- make assumptions
-            fp.TransferSyntaxUID = pydicom.uid.ImplicitVRLittleEndian
-            self._is_little_endian = True
-            self._is_implicit_VR = True
-
-        impl_expl = ("Explicit", "Implicit")[self._is_implicit_VR]
-        big_little = ("Big", "Little")[self._is_little_endian]
-        logger.debug("Using {0:s} VR, {1:s} Endian transfer syntax".format(
-                     impl_expl, big_little))
-
-    def __iter__(self):
-        tags = sorted(self.file_meta_info.keys())
-        for tag in tags:
-            yield self.file_meta_info[tag]
-
-        for data_element in data_element_generator(self.fp,
-                                                   self._is_implicit_VR,
-                                                   self._is_little_endian,
-                                                   stop_when=self.stop_when):
-            yield data_element
-
-
 def data_element_generator(fp,
                            is_implicit_VR,
                            is_little_endian,
@@ -128,34 +41,42 @@ def data_element_generator(fp,
 
     """Create a generator to efficiently return the raw data elements.
 
+    .. note::
+
+        This function is used internally - usually there is no need to call it
+        from user code. To read data from a DICOM file, :func:`dcmread`
+        shall be used instead.
+
     Parameters
     ----------
-    fp : file-like object
-    is_implicit_VR : boolean
-    is_little_endian : boolean
+    fp : file-like
+        The file-like to read from.
+    is_implicit_VR : bool
+        ``True`` if the data is encoded as implicit VR, ``False`` otherwise.
+    is_little_endian : bool
+        ``True`` if the data is encoded as little endian, ``False`` otherwise.
     stop_when : None, callable, optional
-        If None (default), then the whole file is read.
-        A callable which takes tag, VR, length,
-        and returns True or False. If it returns True,
-        read_data_element will just return.
+        If ``None`` (default), then the whole file is read. A callable which
+        takes tag, VR, length, and returns ``True`` or ``False``. If it
+        returns ``True``, ``read_data_element`` will just return.
     defer_size : int, str, None, optional
-        See ``dcmread`` for parameter info.
+        See :func:`dcmread` for parameter info.
     encoding :
         Encoding scheme
     specific_tags : list or None
-        See ``dcmread`` for parameter info.
+        See :func:`dcmread` for parameter info.
 
     Returns
     -------
-    VR : None if implicit VR, otherwise the VR read from the file
-    length :
-        the length as in the DICOM data element (could be
-        DICOM "undefined length" 0xffffffffL)
-    value_bytes :
-        the raw bytes from the DICOM file
-        (not parsed into python types)
-    is_little_endian : boolean
-        True if transfer syntax is little endian; else False.
+    VR : str or None
+        ``None`` if implicit VR, otherwise the VR read from the file.
+    length : int
+        The length of the DICOM data element (could be DICOM "undefined
+        length" ``0xFFFFFFFFL``)
+    value_bytes : bytes or str
+        The raw bytes from the DICOM file (not parsed into Python types)
+    is_little_endian : bool
+        ``True`` if transfer syntax is little endian; else ``False``.
     """
     # Summary of DICOM standard PS3.5-2008 chapter 7:
     # If Implicit VR, data element is:
@@ -272,16 +193,14 @@ def data_element_generator(fp,
                              "Skipping forward to next data element.")
                 fp.seek(fp_tell() + length)
             else:
-                value = fp_read(length)
+                value = (fp_read(length) if length > 0
+                         else empty_value_for_VR(VR, raw=True))
                 if debugging:
-                    dotdot = "   "
-                    if length > 12:
-                        dotdot = "..."
-                    logger_debug("%08x: %-34s %s %r %s" % (value_tell,
-                                                           bytes2hex(
-                                                               value[:12]),
-                                                           dotdot,
-                                                           value[:12], dotdot))
+                    dotdot = "..." if length > 12 else "   "
+                    displayed_value = value[:12] if value else b''
+                    logger_debug("%08x: %-34s %s %r %s" %
+                                 (value_tell, bytes2hex(displayed_value),
+                                  dotdot, displayed_value, dotdot))
 
             # If the tag is (0008,0005) Specific Character Set, then store it
             if tag == BaseTag(0x00080005):
@@ -348,49 +267,114 @@ def data_element_generator(fp,
                                      is_implicit_VR, is_little_endian)
 
 
-def read_dataset(fp, is_implicit_VR, is_little_endian, bytelength=None,
-                 stop_when=None, defer_size=None,
-                 parent_encoding=default_encoding, specific_tags=None):
-    """Return a Dataset instance containing the next dataset in the file.
+def _is_implicit_vr(fp, implicit_vr_is_assumed, is_little_endian, stop_when):
+    """Check if the real VR is explicit or implicit.
 
     Parameters
     ----------
     fp : an opened file object
-    is_implicit_VR : boolean
-        True if file transfer syntax is implicit VR.
-    is_little_endian : boolean
+    implicit_vr_is_assumed : bool
+        True if implicit VR is assumed.
+        If this does not match with the real transfer syntax, a user warning
+        will be issued.
+    is_little_endian : bool
         True if file has little endian transfer syntax.
-    bytelength : int, None, optional
-        None to read until end of file or ItemDeliterTag, else
-        a fixed number of bytes to read
+        Needed to interpret the first tag.
     stop_when : None, optional
-        optional call_back function which can terminate reading.
-        See help for data_element_generator for details
-    defer_size : int, None, optional
-        Size to avoid loading large elements in memory.
-        See ``dcmread`` for more parameter info.
-    parent_encoding :
-        optional encoding to use as a default in case
-        a Specific Character Set (0008,0005) isn't specified
-    specific_tags : list or None
-        See ``dcmread`` for parameter info.
+        Optional call_back function which can terminate reading.
+        Needed to check if the next tag still belongs to the read dataset.
 
     Returns
     -------
-    a Dataset instance
+    True if implicit VR is used, False otherwise.
+    """
+    tag_bytes = fp.read(4)
+    vr = fp.read(2)
+    if len(vr) < 2:
+        return implicit_vr_is_assumed
+
+    # it is sufficient to check if the VR is in valid ASCII range, as it is
+    # extremely unlikely that the tag length accidentally has such a
+    # representation - this would need the first tag to be longer than 16kB
+    # (e.g. it should be > 0x4141 = 16705 bytes)
+    vr1 = ord(vr[0]) if in_py2 else vr[0]
+    vr2 = ord(vr[1]) if in_py2 else vr[1]
+    found_implicit = not (0x40 < vr1 < 0x5B and 0x40 < vr2 < 0x5B)
+
+    if found_implicit != implicit_vr_is_assumed:
+        # first check if the tag still belongs to the dataset if stop_when
+        # is given - if not, the dataset is empty and we just return
+        endian_chr = "<" if is_little_endian else ">"
+        tag = TupleTag(unpack(endian_chr + "HH", tag_bytes))
+        if stop_when is not None and stop_when(tag, vr, 0):
+            return found_implicit
+
+        # got to the real problem - warn or raise depending on config
+        found_vr = 'implicit' if found_implicit else 'explicit'
+        expected_vr = 'implicit' if not found_implicit else 'explicit'
+        message = ('Expected {0} VR, but found {1} VR - using {1} VR for '
+                   'reading'.format(expected_vr, found_vr))
+        if config.enforce_valid_values:
+            raise InvalidDicomError(message)
+        warnings.warn(message, UserWarning)
+    return found_implicit
+
+
+def read_dataset(fp, is_implicit_VR, is_little_endian, bytelength=None,
+                 stop_when=None, defer_size=None,
+                 parent_encoding=default_encoding, specific_tags=None,
+                 at_top_level=True):
+    """Return a :class:`~pydicom.dataset.Dataset` instance containing the next
+    dataset in the file.
+
+    Parameters
+    ----------
+    fp : file-like
+        An opened file-like object.
+    is_implicit_VR : bool
+        ``True`` if file transfer syntax is implicit VR.
+    is_little_endian : bool
+        ``True`` if file has little endian transfer syntax.
+    bytelength : int, None, optional
+        ``None`` to read until end of file or ItemDeliterTag, else a fixed
+        number of bytes to read
+    stop_when : None, optional
+        Optional call_back function which can terminate reading. See help for
+        :func:`data_element_generator` for details
+    defer_size : int, None, optional
+        Size to avoid loading large elements in memory. See :func:`dcmread` for
+        more parameter info.
+    parent_encoding :
+        Optional encoding to use as a default in case (0008,0005) *Specific
+        Character Set* isn't specified.
+    specific_tags : list or None
+        See :func:`dcmread` for parameter info.
+    at_top_level: bool
+        If dataset is top level (not within a sequence).
+        Used to turn off explicit VR heuristic within sequences
+
+    Returns
+    -------
+    dataset.Dataset
+        A Dataset instance.
 
     See Also
     --------
-    pydicom.dataset.Dataset
-        A collection (dictionary) of Dicom `DataElement` instances.
+    :class:`~pydicom.dataset.Dataset`
+        A collection (dictionary) of DICOM
+        :class:`~pydicom.dataelem.DataElement` instances.
     """
     raw_data_elements = dict()
-    fpStart = fp.tell()
+    fp_start = fp.tell()
+    if at_top_level:
+        is_implicit_VR = _is_implicit_vr(
+            fp, is_implicit_VR, is_little_endian, stop_when)
+    fp.seek(fp_start)
     de_gen = data_element_generator(fp, is_implicit_VR, is_little_endian,
                                     stop_when, defer_size, parent_encoding,
                                     specific_tags)
     try:
-        while (bytelength is None) or (fp.tell() - fpStart < bytelength):
+        while (bytelength is None) or (fp.tell() - fp_start < bytelength):
             raw_data_element = next(de_gen)
             # Read data elements. Stop on some errors, but return what was read
             tag = raw_data_element.tag
@@ -401,9 +385,10 @@ def read_dataset(fp, is_implicit_VR, is_little_endian, bytelength=None,
     except StopIteration:
         pass
     except EOFError as details:
-        # XXX is this error visible enough to user code with just logging?
-        logger.error(str(details) + " in file " +
-                     getattr(fp, "name", "<no filename>"))
+        if config.enforce_valid_values:
+            raise
+        msg = str(details) + " in file " + getattr(fp, "name", "<no filename>")
+        warnings.warn(msg, UserWarning)
     except NotImplementedError as details:
         logger.error(details)
 
@@ -419,7 +404,9 @@ def read_dataset(fp, is_implicit_VR, is_little_endian, bytelength=None,
 
 def read_sequence(fp, is_implicit_VR, is_little_endian, bytelength, encoding,
                   offset=0):
-    """Read and return a Sequence -- i.e. a list of Datasets"""
+    """Read and return a :class:`~pydicom.sequence.Sequence` -- i.e. a
+    :class:`list` of :class:`Datasets<pydicom.dataset.Dataset>`.
+    """
 
     seq = []  # use builtin list to start for speed, convert to Sequence at end
     is_undefined_length = False
@@ -444,7 +431,9 @@ def read_sequence(fp, is_implicit_VR, is_little_endian, bytelength, encoding,
 
 def read_sequence_item(fp, is_implicit_VR, is_little_endian, encoding,
                        offset=0):
-    """Read and return a single sequence item, i.e. a Dataset"""
+    """Read and return a single :class:`~pydicom.sequence.Sequence` item, i.e.
+    a :class:`~pydicom.dataset.Dataset`.
+    """
     seq_item_tell = fp.tell() + offset
     if is_little_endian:
         tag_length_format = "<HHL"
@@ -473,11 +462,13 @@ def read_sequence_item(fp, is_implicit_VR, is_little_endian, encoding,
             fp.tell() - 4 + offset, bytes2hex(bytes_read)))
     if length == 0xFFFFFFFF:
         ds = read_dataset(fp, is_implicit_VR, is_little_endian,
-                          bytelength=None, parent_encoding=encoding)
+                          bytelength=None, parent_encoding=encoding,
+                          at_top_level=False)
         ds.is_undefined_length_sequence_item = True
     else:
         ds = read_dataset(fp, is_implicit_VR, is_little_endian, length,
-                          parent_encoding=encoding)
+                          parent_encoding=encoding,
+                          at_top_level=False)
         ds.is_undefined_length_sequence_item = False
         logger.debug("%08x: Finished sequence item" % (fp.tell() + offset,))
     ds.seq_item_tell = seq_item_tell
@@ -488,9 +479,10 @@ def _read_command_set_elements(fp):
     """Return a Dataset containing any Command Set (0000,eeee) elements
     in `fp`.
 
-    Command Set elements are always Implicit VR Little Endian (as per PS3.7
-    Section 6.3). Once any Command Set elements are read `fp` will be
-    positioned at the start of the next group of elements.
+    Command Set elements are always Implicit VR Little Endian (DICOM Standard,
+    Part 7, :dcm:`Section 6.3<part07/sect_6.3.html>`). Once any Command Set
+    elements are read `fp` will be positioned at the start of the next group
+    of elements.
 
     Parameters
     ----------
@@ -499,7 +491,7 @@ def _read_command_set_elements(fp):
 
     Returns
     -------
-    pydicom.dataset.Dataset
+    dataset.Dataset
         The command set elements as a Dataset instance. May be empty if no
         command set elements are present.
     """
@@ -516,9 +508,10 @@ def _read_command_set_elements(fp):
 def _read_file_meta_info(fp):
     """Return a Dataset containing any File Meta (0002,eeee) elements in `fp`.
 
-    File Meta elements are always Explicit VR Little Endian (as per PS3.10
-    Section 7). Once any File Meta elements are read `fp` will be positioned
-    at the start of the next group of elements.
+    File Meta elements are always Explicit VR Little Endian (DICOM Standard,
+    Part 10, :dcm:`Section 7<part10/chapter_7.html>`). Once any File Meta
+    elements are read `fp` will be positioned at the start of the next group
+    of elements.
 
     Parameters
     ----------
@@ -528,7 +521,7 @@ def _read_file_meta_info(fp):
 
     Returns
     -------
-    pydicom.dataset.Dataset
+    dataset.Dataset
         The File Meta elements as a Dataset instance. May be empty if no
         File Meta are present.
     """
@@ -540,7 +533,7 @@ def _read_file_meta_info(fp):
     start_file_meta = fp.tell()
     file_meta = read_dataset(fp, is_implicit_VR=False, is_little_endian=True,
                              stop_when=_not_group_0002)
-    if not file_meta.tags:
+    if not file_meta._dict:
         return file_meta
 
     # Test the file meta for correct interpretation by requesting the first
@@ -589,7 +582,7 @@ def read_preamble(fp, force):
     `fp` should be positioned at the start of the file-like. If the preamble
     and prefix are found then after reading `fp` will be positioned at the
     first byte after the prefix (byte offset 133). If either the preamble or
-    prefix are missing and `force` is True then after reading `fp` will be
+    prefix are missing and `force` is ``True`` then after reading `fp` will be
     positioned at the start of the file-like.
 
     Parameters
@@ -603,13 +596,13 @@ def read_preamble(fp, force):
     -------
     preamble : str/bytes or None
         The 128-byte DICOM preamble will be returned if the appropriate prefix
-        ('DICM') is found at byte offset 128. Returns None if the 'DICM' prefix
-        is not found and `force` is True.
+        ('DICM') is found at byte offset 128. Returns ``None`` if the 'DICM'
+        prefix is not found and `force` is ``True``.
 
     Raises
     ------
     InvalidDicomError
-        If `force` is False and no appropriate header information found.
+        If `force` is ``False`` and no appropriate header information found.
 
     Notes
     -----
@@ -654,22 +647,23 @@ def read_partial(fileobj, stop_when=None, defer_size=None,
     fileobj : a file-like object
         Note that the file will not close when the function returns.
     stop_when :
-        Stop condition. See ``read_dataset`` for more info.
+        Stop condition. See :func:`read_dataset` for more info.
     defer_size : int, str, None, optional
-        See ``dcmread`` for parameter info.
-    force : boolean
-        See ``dcmread`` for parameter info.
+        See :func:`dcmread` for parameter info.
+    force : bool
+        See :func:`dcmread` for parameter info.
     specific_tags : list or None
-        See ``dcmread`` for parameter info.
+        See :func:`dcmread` for parameter info.
 
     Notes
     -----
-    Use ``dcmread`` unless you need to stop on some condition other than
+    Use :func:`dcmread` unless you need to stop on some condition other than
     reaching pixel data.
 
     Returns
     -------
-    FileDataset instance or DicomDir instance.
+    dataset.FileDataset or dicomdir.DicomDir
+        The read dataset.
 
     See Also
     --------
@@ -690,7 +684,8 @@ def read_partial(fileobj, stop_when=None, defer_size=None,
 
     # Check to see if there's anything left to read
     peek = fileobj.read(1)
-    fileobj.seek(-1, 1)
+    if peek != b'':
+        fileobj.seek(-1, 1)
 
     # `filobj` should be positioned at the start of the dataset by this point.
     # Ensure we have appropriate values for `is_implicit_VR` and
@@ -757,10 +752,12 @@ def read_partial(fileobj, stop_when=None, defer_size=None,
                                stop_when=stop_when, defer_size=defer_size,
                                specific_tags=specific_tags)
     except EOFError:
-        pass  # error already logged in read_dataset
+        if config.enforce_valid_values:
+            raise
+        # warning already logged in read_dataset
 
     # Add the command set elements to the dataset (if any)
-    dataset.update(command_set.tags)
+    dataset.update(command_set._dict)
 
     class_uid = file_meta_dataset.get("MediaStorageSOPClassUID", None)
     if class_uid and class_uid.name == "Media Storage Directory Storage":
@@ -779,46 +776,48 @@ def dcmread(fp, defer_size=None, stop_before_pixels=False,
             force=False, specific_tags=None):
     """Read and parse a DICOM dataset stored in the DICOM File Format.
 
-    Read a DICOM dataset stored in accordance with the DICOM File Format
-    (DICOM Standard Part 10 Section 7). If the dataset is not stored in
+    Read a DICOM dataset stored in accordance with the :dcm:`DICOM File
+    Format <part10/chapter_7.html>`. If the dataset is not stored in
     accordance with the File Format (i.e. the preamble and prefix are missing,
-    there are missing required Type 1 File Meta Information Group elements
-    or the entire File Meta Information is missing) then you will have to
-    set `force` to True.
+    there are missing required Type 1 *File Meta Information Group* elements
+    or the entire *File Meta Information* is missing) then you will have to
+    set `force` to ``True``.
 
     Parameters
     ----------
     fp : str or file-like
         Either a file-like object, or a string containing the file name. If a
         file-like object, the caller is responsible for closing it.
-    defer_size : int or str or None
-        If None (default), all elements read into memory. If specified, then if
-        a data element's stored value is larger than `defer_size`, the value is
-        not read into memory until it is accessed in code. Specify an integer
-        (bytes), or a string value with units, e.g. "512 KB", "2 MB".
-    stop_before_pixels : bool
-        If False (default), the full file will be read and parsed. Set True to
-        stop before reading (7FE0,0010) 'Pixel Data' (and all subsequent
-        elements).
-    force : bool
-        If False (default), raises an InvalidDicomError if the file is missing
-        the File Meta Information header. Set to True to force reading even if
-        no File Meta Information header is found.
-    specific_tags : list or None
-        If not None, only the tags in the list are returned. The list
-        elements can be tags or tag names. Note that the tag Specific
-        Character Set is always returned if present - this ensures correct
-        decoding of returned text values.
+    defer_size : int or str or None, optional
+        If ``None`` (default), all elements are read into memory. If specified,
+        then if a data element's stored value is larger than `defer_size`, the
+        value is not read into memory until it is accessed in code. Specify an
+        integer (bytes), or a string value with units, e.g. "512 KB", "2 MB".
+    stop_before_pixels : bool, optional
+        If ``False`` (default), the full file will be read and parsed. Set
+        ``True`` to stop before reading (7FE0,0010) *Pixel Data* (and all
+        subsequent elements).
+    force : bool, optional
+        If ``False`` (default), raises an
+        :class:`~pydicom.errors.InvalidDicomError` if the file is
+        missing the *File Meta Information* header. Set to ``True`` to force
+        reading even if no *File Meta Information* header is found.
+    specific_tags : list or None, optional
+        If not ``None``, only the tags in the list are returned. The list
+        elements can be tags or tag names. Note that the element (0008,0005)
+        *Specific Character Set* is always returned if present - this ensures
+        correct decoding of returned text values.
 
     Returns
     -------
     FileDataset
-        An instance of FileDataset that represents a parsed DICOM file.
+        An instance of :class:`~pydicom.dataset.FileDataset` that represents
+        a parsed DICOM file.
 
     Raises
     ------
     InvalidDicomError
-        If `force` is True and the file is not a valid DICOM file.
+        If `force` is ``True`` and the file is not a valid DICOM file.
 
     See Also
     --------
@@ -889,9 +888,9 @@ read_file = dcmread  # used read_file until pydicom 1.0. Kept for compatibility
 
 
 def read_dicomdir(filename="DICOMDIR"):
-    """Read a DICOMDIR file and return a DicomDir instance.
+    """Read a DICOMDIR file and return a :class:`~pydicom.dicomdir.DicomDir`.
 
-    This is a wrapper around dcmread, which gives a default file name.
+    This is a wrapper around :func:`dcmread` which gives a default file name.
 
     Parameters
     ----------
@@ -931,28 +930,63 @@ def data_element_offset_to_value(is_implicit_VR, VR):
     return offset
 
 
-def read_deferred_data_element(fileobj_type, filename, timestamp,
+def read_deferred_data_element(fileobj_type, filename_or_obj, timestamp,
                                raw_data_elem):
     """Read the previously deferred value from the file into memory
-    and return a raw data element"""
+    and return a raw data element.
+
+    .. note:
+
+        This is called internally by pydicom and will normally not be
+        needed in user code.
+
+    Parameters
+    ----------
+    fileobj_type : type
+        The type of the original file object.
+    filename_or_obj : str or file-like
+        The filename of the original file if one exists, or the file-like
+        object where the data element persists.
+    timestamp : time or None
+        The time the original file has been read, if not a file-like.
+    raw_data_elem : dataelem.RawDataElement
+        The raw data element with no value set.
+
+    Returns
+    -------
+    dataelem.RawDataElement
+        The data element with the value set.
+
+    Raises
+    ------
+    IOError
+        If `filename_or_obj` is ``None``.
+    IOError
+        If `filename_or_obj` is a filename and the corresponding file does
+        not exist.
+    ValueError
+        If the VR or tag of `raw_data_elem` does not match the read value.
+    """
     logger.debug("Reading deferred element %r" % str(raw_data_elem.tag))
     # If it wasn't read from a file, then return an error
-    if filename is None:
+    if filename_or_obj is None:
         raise IOError("Deferred read -- original filename not stored. "
                       "Cannot re-open")
+    is_filename = isinstance(filename_or_obj, compat.string_types)
+
     # Check that the file is the same as when originally read
-    if not os.path.exists(filename):
+    if is_filename and not os.path.exists(filename_or_obj):
         raise IOError(u"Deferred read -- original file "
-                      "{0:s} is missing".format(filename))
+                      "{0:s} is missing".format(filename_or_obj))
     if timestamp is not None:
-        statinfo = os.stat(filename)
+        statinfo = os.stat(filename_or_obj)
         if statinfo.st_mtime != timestamp:
             warnings.warn("Deferred read warning -- file modification time "
                           "has changed.")
 
     # Open the file, position to the right place
-    # fp = self.typefileobj(self.filename, "rb")
-    fp = fileobj_type(filename, 'rb')
+    fp = (fileobj_type(filename_or_obj, 'rb')
+          if is_filename else filename_or_obj)
     is_implicit_VR = raw_data_elem.is_implicit_VR
     is_little_endian = raw_data_elem.is_little_endian
     offset = data_element_offset_to_value(is_implicit_VR, raw_data_elem.VR)
