@@ -1,5 +1,6 @@
 # Copyright 2008-2018 pydicom authors. See LICENSE file for details.
-"""Use the numpy package to convert RLE lossless pixel data to an ndarray.
+"""Use the `numpy <https://numpy.org/>`_ package to convert RLE lossless *Pixel
+Data* to a :class:`numpy.ndarray`.
 
 **Supported transfer syntaxes**
 
@@ -7,9 +8,10 @@
 
 **Supported data**
 
-The RLE handler supports the conversion of data in the (7fe0,0010)
-*Pixel Data* element to a numpy ndarray provided the related Image Pixel module
-elements have values given in the table below.
+The RLE handler supports the conversion of data in the (7FE0,0010)
+*Pixel Data* element to a numpy ndarray provided the related
+:dcm:`Image Pixel<part03/sect_C.7.6.3.html>` module elements have values given
+in the table below.
 
 +------------------------------------------------+--------------+----------+
 | Element                                        | Supported    |          |
@@ -33,7 +35,9 @@ elements have values given in the table below.
 
 """
 
-from struct import unpack
+from itertools import groupby
+from struct import pack, unpack
+import sys
 
 try:
     import numpy as np
@@ -58,17 +62,25 @@ SUPPORTED_TRANSFER_SYNTAXES = [
 
 
 def is_available():
-    """Return True if the handler has its dependencies met."""
+    """Return ``True`` if the handler has its dependencies met."""
     return HAVE_RLE
 
 
 def supports_transfer_syntax(transfer_syntax):
-    """Return True if the handler supports the `transfer_syntax`."""
+    """Return ``True`` if the handler supports the `transfer_syntax`.
+
+    Parameters
+    ----------
+    transfer_syntax : uid.UID
+        The Transfer Syntax UID of the *Pixel Data* that is to be used with
+        the handler.
+    """
     return transfer_syntax in SUPPORTED_TRANSFER_SYNTAXES
 
 
 def needs_to_convert_to_RGB(ds):
-    """Return True if the pixel data should to be converted from YCbCr to RGB.
+    """Return ``True`` if the *Pixel Data* should to be converted from YCbCr to
+    RGB.
 
     This affects JPEG transfer syntaxes.
     """
@@ -76,7 +88,8 @@ def needs_to_convert_to_RGB(ds):
 
 
 def should_change_PhotometricInterpretation_to_RGB(ds):
-    """Return True if the PhotometricInterpretation should be changed to RGB.
+    """Return ``True`` if the *Photometric Interpretation* should be changed
+    to RGB.
 
     This affects JPEG transfer syntaxes.
     """
@@ -84,34 +97,33 @@ def should_change_PhotometricInterpretation_to_RGB(ds):
 
 
 def get_pixeldata(ds, rle_segment_order='>'):
-    """Return an ndarray of the Pixel Data.
+    """Return an :class:`numpy.ndarray` of the *Pixel Data*.
 
     Parameters
     ----------
     ds : dataset.Dataset
-        The DICOM dataset containing an Image Pixel module and the RLE encoded
-        Pixel Data to be converted.
+        The :class:`Dataset` containing an Image Pixel module and the RLE
+        encoded *Pixel Data* to be converted.
     rle_segment_order : str
-        The order of segments used by the RLE decoder when dealing with Bits
-        Allocated > 8. Each RLE segment contains 8-bits of the pixel data,
+        The order of segments used by the RLE decoder when dealing with *Bits
+        Allocated* > 8. Each RLE segment contains 8-bits of the pixel data,
         and segments are supposed to be ordered from MSB to LSB. A value of
-        '>' means interpret the segments as being in big endian order
-        (default) while a value of '<' means interpret the segments as being
-        in little endian order which may be possible if the encoded data is
-        non-conformant.
+        ``'>'`` means interpret the segments as being in big endian order
+        (default) while a value of ``'<'`` means interpret the segments as
+        being in little endian order which may be possible if the encoded data
+        is non-conformant.
 
     Returns
     -------
-    np.ndarray
-        The decoded contents of the Pixel Data element (7FE0,0010) as a 1D
-        array.
+    numpy.ndarray
+        The decoded contents of (7FE0,0010) *Pixel Data* as a 1D array.
 
     Raises
     ------
     AttributeError
-        If the dataset is missing a required element.
+        If `ds` is missing a required element.
     NotImplementedError
-        If the dataset contains pixel data in an unsupported format.
+        If `ds` contains pixel data in an unsupported format.
     ValueError
         If the actual length of the pixel data doesn't match the expected
         length.
@@ -164,6 +176,7 @@ def get_pixeldata(ds, rle_segment_order='>'):
     return arr
 
 
+# RLE decoding functions
 def _parse_rle_header(header):
     """Return a list of byte offsets for the segments in RLE data.
 
@@ -215,7 +228,7 @@ def _parse_rle_header(header):
 
     References
     ----------
-    DICOM Standard, Part 5, Annex G
+    DICOM Standard, Part 5, :dcm:`Annex G<part05/chapter_G.html>`
     """
     if len(header) != 64:
         raise ValueError('The RLE header can only be 64 bytes long')
@@ -360,3 +373,208 @@ def _rle_decode_segment(data):
         pass
 
     return result
+
+
+# RLE encoding functions
+def rle_encode_frame(arr):
+    """Return an :class:`numpy.ndarray` image frame as RLE encoded
+    :class:`bytearray`.
+
+    .. versionadded:: 1.3
+
+    Parameters
+    ----------
+    arr : numpy.ndarray
+        A 2D (if *Samples Per Pixel* = 1) or 3D (if *Samples Per Pixel* = 3)
+        ndarray containing a single frame of the image to be RLE encoded.
+
+    Returns
+    -------
+    bytearray
+        An RLE encoded frame, including the RLE header, following the format
+        specified by the DICOM Standard, Part 5,
+        :dcm:`Annex G<part05/chapter_G.html>`.
+    """
+    shape = arr.shape
+    if len(shape) > 3:
+        # Note: only raises if multi-sample pixel data with multiple frames
+        raise ValueError(
+            "Unable to encode multiple frames at once, please encode one "
+            "frame at a time"
+        )
+
+    # Check the expected number of segments
+    nr_segments = arr.dtype.itemsize
+    if len(shape) == 3:
+        # Number of samples * bytes per sample
+        nr_segments *= shape[-1]
+
+    if nr_segments > 15:
+        raise ValueError(
+            "Unable to encode as the DICOM standard only allows "
+            "a maximum of 15 segments in RLE encoded data"
+        )
+
+    rle_data = bytearray()
+    seg_lengths = []
+    if len(shape) == 3:
+        # Samples Per Pixel > 1
+        for ii in range(arr.shape[-1]):
+            # Need a contiguous array in order to be able to split it up
+            # into byte segments
+            for segment in _rle_encode_plane(arr[..., ii].copy()):
+                rle_data.extend(segment)
+                seg_lengths.append(len(segment))
+    else:
+        # Samples Per Pixel = 1
+        for segment in _rle_encode_plane(arr):
+            rle_data.extend(segment)
+            seg_lengths.append(len(segment))
+
+    # Add the number of segments to the header
+    rle_header = bytearray(pack('<L', len(seg_lengths)))
+
+    # Add the segment offsets, starting at 64 for the first segment
+    # We don't need an offset to any data at the end of the last segment
+    offsets = [64]
+    for ii, length in enumerate(seg_lengths[:-1]):
+        offsets.append(offsets[ii] + length)
+    rle_header.extend(pack('<{}L'.format(len(offsets)), *offsets))
+
+    # Add trailing padding to make up the rest of the header (if required)
+    rle_header.extend(b'\x00' * (64 - len(rle_header)))
+
+    return rle_header + rle_data
+
+
+def _rle_encode_plane(arr):
+    """Yield RLE encoded segments from an image plane as bytearray.
+
+    A plane of N-byte samples must be split into N segments, with each segment
+    containing the same byte of the N-byte samples. For example, in a plane
+    containing 16 bits per sample, the first segment will contain the most
+    significant 8 bits of the samples and the second segment the 8 least
+    significant bits. Each segment is RLE encoded prior to being yielded.
+
+    Parameters
+    ----------
+    arr : numpy.ndarray
+        A 2D ndarray containing a single plane of the image data to be RLE
+        encoded. The dtype of the array should be a multiple of 8 (i.e. uint8,
+        uint32, int16, etc.).
+
+    Yields
+    ------
+    bytearray
+        An RLE encoded segment of the plane, following the format specified
+        by the DICOM Standard, Part 5, :dcm:`Annex G<part05/chapter_G.html>`.
+        The segments are yielded in order from most significant to least.
+    """
+    # Determine the byte order of the array
+    byte_order = arr.dtype.byteorder
+    if byte_order == '=':
+        byte_order = '<' if sys.byteorder == 'little' else '>'
+
+    # Re-view the N-bit array data as N / 8 x uint8s
+    arr8 = arr.view(np.uint8)
+
+    # Reshape the uint8 array data into 1 or more segments and encode
+    bytes_per_sample = arr.dtype.itemsize
+    for ii in range(bytes_per_sample):
+        # If the original byte order is little endian we need to segment
+        #   in reverse order
+        if byte_order == '<':
+            ii = bytes_per_sample - ii - 1
+        segment = arr8.ravel()[ii::bytes_per_sample].reshape(arr.shape)
+
+        yield _rle_encode_segment(segment)
+
+
+def _rle_encode_segment(arr):
+    """Return a 2D numpy ndarray as an RLE encoded bytearray.
+
+    Each row of the image is encoded separately as required by the DICOM
+    Standard.
+
+    Parameters
+    ----------
+    arr : numpy.ndarray
+        A 2D ndarray of 8-bit uint data, representing a Byte Segment as in
+        the DICOM Standard, Part 5, :dcm:`Annex G.2<part05/sect_G.2.html>`.
+
+    Returns
+    -------
+    bytearray
+        The RLE encoded segment, following the format specified by the DICOM
+        Standard. Odd length encoded segments are padded by a trailing ``0x00``
+        to be even length.
+    """
+    out = bytearray()
+    if len(arr.shape) > 1:
+        for row in arr:
+            out.extend(_rle_encode_row(row))
+    else:
+        out.extend(_rle_encode_row(arr))
+
+    # Pad odd length data with a trailing 0x00 byte
+    out.extend(b'\x00' * (len(out) % 2))
+
+    return out
+
+
+def _rle_encode_row(arr):
+    """Return a numpy array as an RLE encoded bytearray.
+
+    Parameters
+    ----------
+    arr : numpy.ndarray
+        A 1D ndarray of 8-bit uint data.
+
+    Returns
+    -------
+    bytes
+        The RLE encoded row, following the format specified by the DICOM
+        Standard, Part 5, :dcm:`Annex G<part05/chapter_G.html>`
+
+    Notes
+    -----
+    * 2-byte repeat runs are always encoded as Replicate Runs rather than
+      only when not preceeded by a Literal Run as suggested by the Standard.
+    """
+    out = []
+    out_append = out.append
+    out_extend = out.extend
+
+    literal = []
+    for key, group in groupby(arr.astype('uint8').tolist()):
+        group = list(group)
+        if len(group) == 1:
+            literal.append(group[0])
+        else:
+            if literal:
+                # Literal runs
+                for ii in range(0, len(literal), 128):
+                    _run = literal[ii:ii + 128]
+                    out_append(len(_run) - 1)
+                    out_extend(_run)
+
+                literal = []
+
+            # Replicate run
+            for ii in range(0, len(group), 128):
+                if len(group[ii:ii + 128]) > 1:
+                    # Replicate run
+                    out_append(257 - len(group[ii:ii + 128]))
+                    out_append(group[0])
+                else:
+                    # Literal run only if last replicate part is length 1
+                    out_append(0)
+                    out_append(group[0])
+
+    # Final literal run if literal isn't followed by a replicate run
+    for ii in range(0, len(literal), 128):
+        _run = literal[ii:ii + 128]
+        out_append(len(_run) - 1)
+        out_extend(_run)
+
+    return pack('{}B'.format(len(out)), *out)
