@@ -31,6 +31,7 @@ from sys import byteorder
 import pytest
 
 import pydicom
+from pydicom import config
 from pydicom.data import get_testdata_files
 from pydicom.dataset import Dataset
 from pydicom.filereader import dcmread
@@ -46,6 +47,7 @@ from pydicom.uid import (
 
 try:
     import numpy as np
+
     HAVE_NP = True
 except ImportError:
     HAVE_NP = False
@@ -56,13 +58,11 @@ try:
         get_pixeldata,
         unpack_bits,
         pack_bits,
-        get_expected_length,
         pixel_dtype,
     )
 except ImportError:
     NP_HANDLER = None
 
-
 # Paths to the test datasets
 # IMPL: Implicit VR Little Endian
 # EXPL: Explicit VR Little Endian
@@ -92,6 +92,10 @@ EXPL_8_3_1F = get_testdata_files("SC_rgb.dcm")[0]
 EXPB_8_3_1F = get_testdata_files("SC_rgb_expb.dcm")[0]
 # 8/8, 3 samples/pixel, 1 frame, 3 x 3
 EXPL_8_3_1F_ODD = get_testdata_files('SC_rgb_small_odd.dcm')[0]
+# 8/8, 3 sample/pixel, 1 frame, YBR_FULL_422
+EXPL_8_3_1F_YBR422 = get_testdata_files('SC_ybr_full_422_uncompressed.dcm')[0]
+# 8/8, 3 sample/pixel, 1 frame, YBR_FULL
+EXPL_8_3_1F_YBR = get_testdata_files('SC_ybr_full_uncompressed.dcm')[0]
 # 8/8, 3 sample/pixel, 2 frame
 EXPL_8_3_2F = get_testdata_files("SC_rgb_2frame.dcm")[0]
 EXPB_8_3_2F = get_testdata_files("SC_rgb_expb_2frame.dcm")[0]
@@ -99,6 +103,8 @@ EXPB_8_3_2F = get_testdata_files("SC_rgb_expb_2frame.dcm")[0]
 IMPL_16_1_1F = get_testdata_files("MR_small_implicit.dcm")[0]
 EXPL_16_1_1F = get_testdata_files("MR_small.dcm")[0]
 EXPB_16_1_1F = get_testdata_files("MR_small_expb.dcm")[0]
+# Pixel Data with 128 bytes trailing padding
+EXPL_16_1_1F_PAD = get_testdata_files("MR_small_padded.dcm")[0]
 # 16/12, 1 sample/pixel, 10 frame
 EXPL_16_1_10F = get_testdata_files("emri_small.dcm")[0]
 EXPB_16_1_10F = get_testdata_files("emri_small_big_endian.dcm")[0]
@@ -140,6 +146,9 @@ JPEG_2K_LOSSLESS = get_testdata_files("emri_small_jpeg_2k_lossless.dcm")[0]
 JPEG_2K = get_testdata_files("JPEG2000.dcm")[0]
 # RLE Lossless
 RLE = get_testdata_files("MR_small_RLE.dcm")[0]
+# No Image Pixel module
+NO_PIXEL = get_testdata_files("rtplan.dcm")[0]
+
 
 # Transfer Syntaxes (non-retired + Explicit VR Big Endian)
 SUPPORTED_SYNTAXES = [
@@ -171,11 +180,15 @@ REFERENCE_DATA_UNSUPPORTED = [
     (RLE, ('1.2.840.10008.1.2.5', 'CompressedSamples^MR1')),
 ]
 
+SUPPORTED_HANDLER_NAMES = (
+    'numpy', 'NumPy', 'np', 'np_handler', 'numpy_handler'
+)
 
 # Numpy and the numpy handler are unavailable
 @pytest.mark.skipif(HAVE_NP, reason='Numpy is available')
 class TestNoNumpy_NoNumpyHandler(object):
     """Tests for handling datasets without numpy and the handler."""
+
     def setup(self):
         """Setup the environment."""
         self.original_handlers = pydicom.config.pixel_data_handlers
@@ -228,11 +241,19 @@ class TestNoNumpy_NoNumpyHandler(object):
                                match="UID of '{}'".format(uid)):
                 ds.pixel_array
 
+    def test_using_numpy_handler_raises(self):
+        ds = dcmread(EXPL_16_1_1F)
+        msg = ("The pixel data handler 'numpy' is not available on your "
+               "system. Please refer to the pydicom documentation*")
+        with pytest.raises(RuntimeError, match=msg):
+            ds.decompress('numpy')
+
 
 # Numpy unavailable and the numpy handler is available
 @pytest.mark.skipif(HAVE_NP, reason='Numpy is available')
 class TestNoNumpy_NumpyHandler(object):
     """Tests for handling datasets without numpy and the handler."""
+
     def setup(self):
         """Setup the environment."""
         self.original_handlers = pydicom.config.pixel_data_handlers
@@ -303,6 +324,7 @@ class TestNoNumpy_NumpyHandler(object):
 @pytest.mark.skipif(not HAVE_NP, reason='Numpy is unavailable')
 class TestNumpy_NoNumpyHandler(object):
     """Tests for handling datasets without the handler."""
+
     def setup(self):
         """Setup the environment."""
         self.original_handlers = pydicom.config.pixel_data_handlers
@@ -366,11 +388,11 @@ MATCHING_DATASETS = [
     (EXPL_8_3_2F, EXPB_8_3_2F),
     (EXPL_16_1_1F, EXPB_16_1_1F),
     (EXPL_16_1_10F, EXPB_16_1_10F),
-    # (EXPL_16_3_1F, EXPB_16_3_1F),  # Not supported yet
+    (EXPL_16_3_1F, EXPB_16_3_1F),
     (EXPL_16_3_2F, EXPB_16_3_2F),
     (IMPL_32_1_1F, EXPB_32_1_1F),
     (IMPL_32_1_15F, EXPB_32_1_15F),
-    # (EXPL_32_3_1F, EXPB_32_3_1F),  # Not supported yet
+    (EXPL_32_3_1F, EXPB_32_3_1F),
     (EXPL_32_3_2F, EXPB_32_3_2F)
 ]
 
@@ -382,16 +404,17 @@ REFERENCE_DATA_LITTLE = [
     (EXPL_1_1_3F, (EXPL, 1, 1, 0, 3, (3, 512, 512), 'uint8')),
     (EXPL_8_1_1F, (EXPL, 8, 1, 0, 1, (600, 800), 'uint8')),
     (EXPL_8_3_1F_ODD, (EXPL, 8, 3, 0, 1, (3, 3, 3), 'uint8')),
+    (EXPL_8_3_1F_YBR422, (EXPL, 8, 3, 0, 1, (100, 100, 3), 'uint8')),
     (EXPL_8_1_2F, (EXPL, 8, 1, 0, 2, (2, 600, 800), 'uint8')),
     (EXPL_8_3_1F, (EXPL, 8, 3, 0, 1, (100, 100, 3), 'uint8')),
     (EXPL_8_3_2F, (EXPL, 8, 3, 0, 2, (2, 100, 100, 3), 'uint8')),
     (EXPL_16_1_1F, (EXPL, 16, 1, 1, 1, (64, 64), 'int16')),
     (EXPL_16_1_10F, (EXPL, 16, 1, 0, 10, (10, 64, 64), 'uint16')),
-    # (EXPL_16_3_1F, (EXPL, 16, 3, 0, 1, (100, 100, 3), 'uint16')),
+    (EXPL_16_3_1F, (EXPL, 16, 3, 0, 1, (100, 100, 3), 'uint16')),
     (EXPL_16_3_2F, (EXPL, 16, 3, 0, 2, (2, 100, 100, 3), 'uint16')),
     (IMPL_32_1_1F, (IMPL, 32, 1, 0, 1, (10, 10), 'uint32')),
     (IMPL_32_1_15F, (IMPL, 32, 1, 0, 15, (15, 10, 10), 'uint32')),
-    # (EXPL_32_3_1F, (EXPL, 32, 3, 0, 1, (100, 100, 3), 'uint32')),
+    (EXPL_32_3_1F, (EXPL, 32, 3, 0, 1, (100, 100, 3), 'uint32')),
     (EXPL_32_3_2F, (EXPL, 32, 3, 0, 2, (2, 100, 100, 3), 'uint32')),
 ]
 
@@ -399,6 +422,7 @@ REFERENCE_DATA_LITTLE = [
 @pytest.mark.skipif(not HAVE_NP, reason='Numpy is not available')
 class TestNumpy_NumpyHandler(object):
     """Tests for handling Pixel Data with the handler."""
+
     def setup(self):
         """Setup the test datasets and the environment."""
         self.original_handlers = pydicom.config.pixel_data_handlers
@@ -450,6 +474,17 @@ class TestNumpy_NumpyHandler(object):
         # Reset
         NP_HANDLER.needs_to_convert_to_RGB = orig_fn
 
+    def test_dataset_pixel_array_no_pixels(self):
+        """Test good exception message if no pixel data in dataset."""
+        ds = dcmread(NO_PIXEL)
+        msg = (
+            r"Unable to convert the pixel data: one of Pixel Data, Float "
+            r"Pixel Data or Double Float Pixel Data must be present in the "
+            r"dataset"
+        )
+        with pytest.raises(AttributeError, match=msg):
+            ds.pixel_array
+
     @pytest.mark.parametrize("fpath, data", REFERENCE_DATA_UNSUPPORTED)
     def test_can_access_unsupported_dataset(self, fpath, data):
         """Test can read and access elements in unsupported datasets."""
@@ -472,6 +507,16 @@ class TestNumpy_NumpyHandler(object):
         assert (1, -10, 1) == tuple(arr[300, 491:494])
         assert 0 == arr[-1].min() == arr[-1].max()
 
+    @pytest.mark.parametrize("handler_name", SUPPORTED_HANDLER_NAMES)
+    def test_decompress_using_handler(self, handler_name):
+        """Test different possibilities for the numpy handler name."""
+        ds = dcmread(EXPL_8_1_1F)
+        ds.decompress(handler_name)
+        assert (600, 800) == ds.pixel_array.shape
+        assert 244 == ds.pixel_array[0].min() == ds.pixel_array[0].max()
+        assert (1, 246, 1) == tuple(ds.pixel_array[300, 491:494])
+        assert 0 == ds.pixel_array[-1].min() == ds.pixel_array[-1].max()
+
     def test_pixel_array_16bit_un_signed(self):
         """Test pixel_array for 16-bit unsigned -> signed."""
         ds = dcmread(EXPL_16_3_1F)
@@ -532,7 +577,7 @@ class TestNumpy_NumpyHandler(object):
             assert (1, 246, 1) == tuple(arr[0, 300, 491:494])
             assert 0 == arr[0, -1].min() == arr[0, -1].max()
             # Frame 2 is frame 1 inverted
-            assert np.array_equal((2**ds.BitsAllocated - 1) - arr[1], arr[0])
+            assert np.array_equal((2 ** ds.BitsAllocated - 1) - arr[1], arr[0])
 
     def test_8bit_3sample_1frame_odd_size(self):
         """Test pixel_array for odd sized (3x3) pixel data."""
@@ -552,6 +597,31 @@ class TestNumpy_NumpyHandler(object):
                 [158, 158, 158], [158, 158, 158], [158, 158, 158]
             ]
 
+    def test_8bit_3sample_1frame_ybr422(self):
+        """Test pixel_array for YBR_FULL_422 pixel data."""
+        ds = dcmread(EXPL_8_3_1F_YBR422)
+        assert ds.PhotometricInterpretation == 'YBR_FULL_422'
+        arr = ds.pixel_array
+
+        # Check resampling
+        assert [
+                   [76, 85, 255],
+                   [76, 85, 255],
+                   [76, 85, 255],
+                   [76, 85, 255]
+               ] == arr[0:4, 0, :].tolist()
+        # Check values
+        assert (76, 85, 255) == tuple(arr[5, 50, :])
+        assert (166, 106, 193) == tuple(arr[15, 50, :])
+        assert (150, 46, 20) == tuple(arr[25, 50, :])
+        assert (203, 86, 75) == tuple(arr[35, 50, :])
+        assert (29, 255, 107) == tuple(arr[45, 50, :])
+        assert (142, 193, 118) == tuple(arr[55, 50, :])
+        assert (0, 128, 128) == tuple(arr[65, 50, :])
+        assert (64, 128, 128) == tuple(arr[75, 50, :])
+        assert (192, 128, 128) == tuple(arr[85, 50, :])
+        assert (255, 128, 128) == tuple(arr[95, 50, :])
+
     def test_8bit_3sample_1frame(self):
         """Test pixel_array for 8-bit, 3 sample/pixel, 1 frame."""
         # Check supported syntaxes
@@ -596,7 +666,7 @@ class TestNumpy_NumpyHandler(object):
             assert (192, 192, 192) == tuple(frame[85, 50, :])
             assert (255, 255, 255) == tuple(frame[95, 50, :])
             # Frame 2 is frame 1 inverted
-            assert np.array_equal((2**ds.BitsAllocated - 1) - arr[1], arr[0])
+            assert np.array_equal((2 ** ds.BitsAllocated - 1) - arr[1], arr[0])
 
     # Little endian datasets
     @pytest.mark.parametrize('fpath, data', REFERENCE_DATA_LITTLE)
@@ -620,6 +690,9 @@ class TestNumpy_NumpyHandler(object):
             nr_frames = getattr(ds, 'NumberOfFrames', 1)
             # Odd sized data is padded by a final 0x00 byte
             size = ds.Rows * ds.Columns * nr_frames * data[1] / 8 * data[2]
+            # YBR_FULL_422 data is 2/3rds usual size
+            if ds.PhotometricInterpretation == 'YBR_FULL_422':
+                size = size // 3 * 2
             assert len(ds.PixelData) == size + size % 2
             if size % 2:
                 assert ds.PixelData[-1] == b'\x00'[0]
@@ -716,6 +789,42 @@ class TestNumpy_NumpyHandler(object):
             assert (422, 319, 361) == tuple(arr[0, 31:34])
             assert (366, 363, 322) == tuple(arr[31, :3])
             assert (1369, 1129, 862) == tuple(arr[-1, -3:])
+            # Last pixel
+            assert 862 == arr[-1, -1]
+
+    def test_little_16bit_1sample_1frame_padded(self):
+        """Test with padded little 16-bit, 1 sample/pixel, 1 frame."""
+        ds = dcmread(EXPL_16_1_1F_PAD)
+        assert ds.file_meta.TransferSyntaxUID == ExplicitVRLittleEndian
+        assert ds.BitsAllocated == 16
+        assert ds.SamplesPerPixel == 1
+        assert ds.PixelRepresentation == 1
+        nr_frames = getattr(ds, 'NumberOfFrames', 1)
+        assert nr_frames == 1
+
+        # Odd sized data is padded by a final 0x00 byte
+        size = ds.Rows * ds.Columns * nr_frames * 16 / 8 * ds.SamplesPerPixel
+        # Has excess padding
+        assert len(ds.PixelData) > size + size % 2
+
+        msg = (
+            r"The length of the pixel data in the dataset \(8320 bytes\) "
+            r"indicates it contains excess padding. 128 bytes will be "
+            r"removed from the end of the data"
+        )
+        with pytest.warns(UserWarning, match=msg):
+            arr = ds.pixel_array
+
+        assert (64, 64) == arr.shape
+        assert arr.dtype == 'int16'
+
+        assert arr.flags.writeable
+
+        assert (422, 319, 361) == tuple(arr[0, 31:34])
+        assert (366, 363, 322) == tuple(arr[31, :3])
+        assert (1369, 1129, 862) == tuple(arr[-1, -3:])
+        # Last pixel
+        assert 862 == arr[-1, -1]
 
     def test_little_16bit_1sample_10frame(self):
         """Test pixel_array for little 16-bit, 1 sample/pixel, 10 frame."""
@@ -783,7 +892,7 @@ class TestNumpy_NumpyHandler(object):
             assert (49344, 49344, 49344) == tuple(arr[0, 85, 50, :])
             assert (65535, 65535, 65535) == tuple(arr[0, 95, 50, :])
             # Frame 2 is frame 1 inverted
-            assert np.array_equal((2**ds.BitsAllocated - 1) - arr[1], arr[0])
+            assert np.array_equal((2 ** ds.BitsAllocated - 1) - arr[1], arr[0])
 
     def test_little_32bit_1sample_1frame(self):
         """Test pixel_array for little 32-bit, 1 sample/pixel, 1 frame."""
@@ -877,7 +986,73 @@ class TestNumpy_NumpyHandler(object):
                 arr[0, 95, 50, :]
             )
             # Frame 2 is frame 1 inverted
-            assert np.array_equal((2**ds.BitsAllocated - 1) - arr[1], arr[0])
+            assert np.array_equal((2 ** ds.BitsAllocated - 1) - arr[1], arr[0])
+
+    def test_little_32bit_float_1frame(self):
+        """Test pixel_array for float pixel data, 1 frame."""
+        ds = dcmread(IMPL_32_1_1F)
+        ds.FloatPixelData = ds.PixelData
+        del ds.PixelData
+        for uid in SUPPORTED_SYNTAXES[:3]:
+            ds.file_meta.TransferSyntaxUID = uid
+
+            arr = ds.pixel_array
+
+            assert arr.flags.writeable
+            assert (10, 10) == arr.shape
+            assert 1.75e-39 == pytest.approx(arr[0, 0], abs=0.01e-39)
+            assert 1.44e-39 == pytest.approx(arr[4, 3], abs=0.01e-39)
+            assert 1.13e-39 == pytest.approx(arr[-1, -3], abs=0.01e-39)
+
+    def test_little_32bit_float_15frame(self):
+        """Test pixel_array for float pixel data, 15 frames."""
+        ds = dcmread(IMPL_32_1_15F)
+        ds.FloatPixelData = ds.PixelData
+        del ds.PixelData
+        for uid in SUPPORTED_SYNTAXES[:3]:
+            ds.file_meta.TransferSyntaxUID = uid
+
+            arr = ds.pixel_array
+
+            assert arr.flags.writeable
+            assert (15, 10, 10) == arr.shape
+            assert 1.75e-39 == pytest.approx(arr[0, 0, 0], abs=0.01e-39)
+            assert 1.44e-39 == pytest.approx(arr[0, 4, 3], abs=0.01e-39)
+            assert 1.13e-39 == pytest.approx(arr[0, -1, -3], abs=0.01e-39)
+
+    def test_little_64bit_float_1frame(self):
+        """Test pixel_array for double float pixel data, 1 frame."""
+        ds = dcmread(IMPL_32_1_1F)
+        ds.DoubleFloatPixelData = ds.PixelData + ds.PixelData
+        del ds.PixelData
+        ds.BitsAllocated = 64
+        for uid in SUPPORTED_SYNTAXES[:3]:
+            ds.file_meta.TransferSyntaxUID = uid
+
+            arr = ds.pixel_array
+
+            assert arr.flags.writeable
+            assert (10, 10) == arr.shape
+            assert 2.65e-308 == pytest.approx(arr[0, 0], abs=0.01e-308)
+            assert 1.80e-308 == pytest.approx(arr[4, 3], abs=0.01e-308)
+            assert 1.69e-308 == pytest.approx(arr[-1, -3], abs=0.01e-308)
+
+    def test_little_64bit_float_15frame(self):
+        """Test pixel_array for double float pixel data, 15 frames."""
+        ds = dcmread(IMPL_32_1_15F)
+        ds.DoubleFloatPixelData = ds.PixelData + ds.PixelData
+        del ds.PixelData
+        ds.BitsAllocated = 64
+        for uid in SUPPORTED_SYNTAXES[:3]:
+            ds.file_meta.TransferSyntaxUID = uid
+
+            arr = ds.pixel_array
+
+            assert arr.flags.writeable
+            assert (15, 10, 10) == arr.shape
+            assert 2.65e-308 == pytest.approx(arr[0, 0, 0], abs=0.01e-308)
+            assert 1.80e-308 == pytest.approx(arr[0, 4, 3], abs=0.01e-308)
+            assert 1.69e-308 == pytest.approx(arr[0, -1, -3], abs=0.01e-308)
 
     # Big endian datasets
     @pytest.mark.parametrize('little, big', MATCHING_DATASETS)
@@ -900,6 +1075,7 @@ class TestNumpy_NumpyHandler(object):
         ds.BitsAllocated = 16
         ds.PixelRepresentation = 0
         ds.SamplesPerPixel = 1
+        ds.PhotometricInterpretation = 'MONOCHROME2'
         arr = np.ones((10, 10), dtype='uint16')
         ds.PixelData = arr.tobytes()
 
@@ -924,7 +1100,28 @@ class TestNumpy_GetPixelData(object):
         ds = dcmread(EXPL_16_1_1F)
         del ds.PixelData
         assert 'PixelData' not in ds
-        with pytest.raises(AttributeError, match=' dataset: PixelData'):
+        assert 'FloatPixelData' not in ds
+        assert 'DoubleFloatPixelData' not in ds
+        msg = (
+            r"Unable to convert the pixel data: one of Pixel Data, Float "
+            r"Pixel Data or Double Float Pixel Data must be present in "
+            r"the dataset"
+        )
+        with pytest.raises(AttributeError, match=msg):
+            get_pixeldata(ds)
+
+    def test_missing_required_elem(self):
+        """Tet get_pixeldata raises if dataset missing required element."""
+        ds = dcmread(EXPL_16_1_1F)
+        del ds.BitsAllocated
+        del ds.Rows
+        del ds.SamplesPerPixel
+        msg = (
+            r"Unable to convert the pixel data as the following required "
+            r"elements are missing from the dataset: BitsAllocated, Rows, "
+            r"SamplesPerPixel"
+        )
+        with pytest.raises(AttributeError, match=msg):
             get_pixeldata(ds)
 
     def test_unknown_pixel_representation_raises(self):
@@ -949,20 +1146,26 @@ class TestNumpy_GetPixelData(object):
         # Too short
         ds.PixelData = ds.PixelData[:-1]
         msg = (
-            r"The length of the pixel data in the dataset doesn't match the "
-            r"expected amount \(479999 vs. 480000 bytes\). The dataset may be "
-            r"corrupted or there may be an issue with the pixel data handler."
+            r"The length of the pixel data in the dataset \(479999 bytes\) "
+            r"doesn't match the expected length \(480000 bytes\). "
+            r"The dataset may be corrupted or there may be an issue "
+            r"with the pixel data handler."
         )
         with pytest.raises(ValueError, match=msg):
             get_pixeldata(ds)
 
-        # Too long
-        ds.PixelData += b'\x00\x00'
-        with pytest.raises(ValueError, match=r"480001 vs. 480000 bytes"):
+    def test_missing_padding_warns(self):
+        """A warning shall be issued if the padding for odd data is missing."""
+        ds = dcmread(EXPL_8_3_1F_ODD)
+        # remove the padding byte
+        ds.PixelData = ds.PixelData[:-1]
+        msg = r"The odd length pixel data is missing a trailing padding byte"
+        with pytest.warns(UserWarning, match=msg):
             get_pixeldata(ds)
 
     def test_change_photometric_interpretation(self):
         """Test get_pixeldata changes PhotometricInterpretation if required."""
+
         def to_rgb(ds):
             """Override the original function that returned False"""
             return True
@@ -1006,6 +1209,63 @@ class TestNumpy_GetPixelData(object):
         arr = get_pixeldata(ds, read_only=True)
         assert arr.flags.writeable
 
+    def test_ybr422_excess_padding(self):
+        """Test YBR data with excess padding."""
+        ds = dcmread(EXPL_8_3_1F_YBR422)
+        assert ds.PhotometricInterpretation == 'YBR_FULL_422'
+        ds.PixelData += b'\x00\x00\x00\x00'
+        msg = (
+            r"The length of the pixel data in the dataset \(20004 bytes\) "
+            r"indicates it contains excess padding. 4 bytes will be removed "
+            r"from the end of the data"
+        )
+        with pytest.warns(UserWarning, match=msg):
+            arr = ds.pixel_array
+
+        assert (76, 85, 255) == tuple(arr[5, 50, :])
+        assert (166, 106, 193) == tuple(arr[15, 50, :])
+        assert (150, 46, 20) == tuple(arr[25, 50, :])
+        assert (203, 86, 75) == tuple(arr[35, 50, :])
+        assert (29, 255, 107) == tuple(arr[45, 50, :])
+        assert (142, 193, 118) == tuple(arr[55, 50, :])
+        assert (0, 128, 128) == tuple(arr[65, 50, :])
+        assert (64, 128, 128) == tuple(arr[75, 50, :])
+        assert (192, 128, 128) == tuple(arr[85, 50, :])
+        assert (255, 128, 128) == tuple(arr[95, 50, :])
+
+    def test_ybr422_wrong_interpretation(self):
+        """Test YBR data with wrong Photometric Interpretation."""
+        ds = dcmread(EXPL_8_3_1F_YBR)
+        assert ds.PhotometricInterpretation == 'YBR_FULL'
+        assert len(ds.PixelData) == 30000
+        ds.PhotometricInterpretation = 'YBR_FULL_422'
+        msg = r"The Photometric Interpretation of the dataset is YBR_FULL_422"
+        with pytest.warns(UserWarning, match=msg):
+            arr = ds.pixel_array
+
+        # Resulting data will be nonsense but of correct shape
+        assert (100, 100, 3) == arr.shape
+
+    def test_float_pixel_data(self):
+        """Test handling of Float Pixel Data."""
+        # Only 1 sample per pixel allowed
+        ds = dcmread(IMPL_32_1_1F)
+        ds.FloatPixelData = ds.PixelData
+        del ds.PixelData
+        assert 32 == ds.BitsAllocated
+        arr = get_pixeldata(ds)
+        assert 'float32' == arr.dtype
+
+    def test_double_float_pixel_data(self):
+        """Test handling of Double Float Pixel Data."""
+        # Only 1 sample per pixel allowed
+        ds = dcmread(IMPL_32_1_1F)
+        ds.DoubleFloatPixelData = ds.PixelData + ds.PixelData
+        del ds.PixelData
+        ds.BitsAllocated = 64
+        arr = get_pixeldata(ds)
+        assert 'float64' == arr.dtype
+
 
 REFERENCE_PACK_UNPACK = [
     (b'', []),
@@ -1036,6 +1296,7 @@ REFERENCE_PACK_UNPACK = [
 @pytest.mark.skipif(not HAVE_NP, reason="Numpy is not available")
 class TestNumpy_UnpackBits(object):
     """Tests for numpy_handler.unpack_bits."""
+
     @pytest.mark.parametrize('input, output', REFERENCE_PACK_UNPACK)
     def test_unpack(self, input, output):
         """Test unpacking data."""
@@ -1066,6 +1327,7 @@ REFERENCE_PACK_PARTIAL = [
 @pytest.mark.skipif(not HAVE_NP, reason="Numpy is not available")
 class TestNumpy_PackBits(object):
     """Tests for numpy_handler.pack_bits."""
+
     @pytest.mark.parametrize('output, input', REFERENCE_PACK_UNPACK)
     def test_pack(self, input, output):
         """Test packing data."""
@@ -1098,105 +1360,3 @@ class TestNumpy_PackBits(object):
         arr = ds.pixel_array
         arr = arr.ravel()
         assert ds.PixelData == pack_bits(arr)
-
-
-REFERENCE_LENGTH = [
-    # (frames, rows, cols, samples), bit depth, result in (bytes, pixels)
-    # No 'NumberOfFrames' in dataset
-    ((0, 0, 0, 0), 1, (0, 0)),
-    ((0, 1, 1, 1), 1, (1, 1)),  # 1 bit -> 1 byte
-    ((0, 1, 1, 3), 1, (1, 3)),  # 3 bits -> 1 byte
-    ((0, 1, 3, 3), 1, (2, 9)),  # 9 bits -> 2 bytes
-    ((0, 2, 2, 1), 1, (1, 4)),  # 4 bits -> 1 byte
-    ((0, 2, 4, 1), 1, (1, 8)),  # 8 bits -> 1 byte
-    ((0, 3, 3, 1), 1, (2, 9)),  # 9 bits -> 2 bytes
-    ((0, 512, 512, 1), 1, (32768, 262144)),  # Typical length
-    ((0, 512, 512, 3), 1, (98304, 786432)),
-    ((0, 0, 0, 0), 8, (0, 0)),
-    ((0, 1, 1, 1), 8, (1, 1)),  # Odd length
-    ((0, 9, 1, 1), 8, (9, 9)),  # Odd length
-    ((0, 1, 2, 1), 8, (2, 2)),  # Even length
-    ((0, 512, 512, 1), 8, (262144, 262144)),
-    ((0, 512, 512, 3), 8, (786432, 786432)),
-    ((0, 0, 0, 0), 16, (0, 0)),
-    ((0, 1, 1, 1), 16, (2, 1)),  # 16 bit data can't be odd length
-    ((0, 1, 2, 1), 16, (4, 2)),
-    ((0, 512, 512, 1), 16, (524288, 262144)),
-    ((0, 512, 512, 3), 16, (1572864, 786432)),
-    ((0, 0, 0, 0), 32, (0, 0)),
-    ((0, 1, 1, 1), 32, (4, 1)),  # 32 bit data can't be odd length
-    ((0, 1, 2, 1), 32, (8, 2)),
-    ((0, 512, 512, 1), 32, (1048576, 262144)),
-    ((0, 512, 512, 3), 32, (3145728, 786432)),
-    # NumberOfFrames odd
-    ((3, 0, 0, 0), 1, (0, 0)),
-    ((3, 1, 1, 1), 1, (1, 3)),
-    ((3, 1, 1, 3), 1, (2, 9)),
-    ((3, 1, 3, 3), 1, (4, 27)),
-    ((3, 2, 4, 1), 1, (3, 24)),
-    ((3, 2, 2, 1), 1, (2, 12)),
-    ((3, 3, 3, 1), 1, (4, 27)),
-    ((3, 512, 512, 1), 1, (98304, 786432)),
-    ((3, 512, 512, 3), 1, (294912, 2359296)),
-    ((3, 0, 0, 0), 8, (0, 0)),
-    ((3, 1, 1, 1), 8, (3, 3)),
-    ((3, 9, 1, 1), 8, (27, 27)),
-    ((3, 1, 2, 1), 8, (6, 6)),
-    ((3, 512, 512, 1), 8, (786432, 786432)),
-    ((3, 512, 512, 3), 8, (2359296, 2359296)),
-    ((3, 0, 0, 0), 16, (0, 0)),
-    ((3, 512, 512, 1), 16, (1572864, 786432)),
-    ((3, 512, 512, 3), 16, (4718592, 2359296)),
-    ((3, 0, 0, 0), 32, (0, 0)),
-    ((3, 512, 512, 1), 32, (3145728, 786432)),
-    ((3, 512, 512, 3), 32, (9437184, 2359296)),
-    # NumberOfFrames even
-    ((4, 0, 0, 0), 1, (0, 0)),
-    ((4, 1, 1, 1), 1, (1, 4)),
-    ((4, 1, 1, 3), 1, (2, 12)),
-    ((4, 1, 3, 3), 1, (5, 36)),
-    ((4, 2, 4, 1), 1, (4, 32)),
-    ((4, 2, 2, 1), 1, (2, 16)),
-    ((4, 3, 3, 1), 1, (5, 36)),
-    ((4, 512, 512, 1), 1, (131072, 1048576)),
-    ((4, 512, 512, 3), 1, (393216, 3145728)),
-    ((4, 0, 0, 0), 8, (0, 0)),
-    ((4, 512, 512, 1), 8, (1048576, 1048576)),
-    ((4, 512, 512, 3), 8, (3145728, 3145728)),
-    ((4, 0, 0, 0), 16, (0, 0)),
-    ((4, 512, 512, 1), 16, (2097152, 1048576)),
-    ((4, 512, 512, 3), 16, (6291456, 3145728)),
-    ((4, 0, 0, 0), 32, (0, 0)),
-    ((4, 512, 512, 1), 32, (4194304, 1048576)),
-    ((4, 512, 512, 3), 32, (12582912, 3145728)),
-]
-
-
-@pytest.mark.skipif(not HAVE_NP, reason="Numpy is not available")
-class TestNumpy_GetExpectedLength(object):
-    """Tests for numpy_handler.get_expected_length."""
-    @pytest.mark.parametrize('shape, bits, length', REFERENCE_LENGTH)
-    def test_length_in_bytes(self, shape, bits, length):
-        """Test get_expected_length(ds, unit='bytes')."""
-        ds = Dataset()
-        ds.Rows = shape[1]
-        ds.Columns = shape[2]
-        ds.BitsAllocated = bits
-        if shape[0] != 0:
-            ds.NumberOfFrames = shape[0]
-        ds.SamplesPerPixel = shape[3]
-
-        assert length[0] == get_expected_length(ds, unit='bytes')
-
-    @pytest.mark.parametrize('shape, bits, length', REFERENCE_LENGTH)
-    def test_length_in_pixels(self, shape, bits, length):
-        """Test get_expected_length(ds, unit='pixels')."""
-        ds = Dataset()
-        ds.Rows = shape[1]
-        ds.Columns = shape[2]
-        ds.BitsAllocated = bits
-        if shape[0] != 0:
-            ds.NumberOfFrames = shape[0]
-        ds.SamplesPerPixel = shape[3]
-
-        assert length[1] == get_expected_length(ds, unit='pixels')
